"""
Google Translate
"""

import os
os.system('pip install googletrans')

from googletrans import Translator
translator = Translator()


def to_en(text):
    t = translator.translate(text, dest='en')
    return t.text

def to_th(text):
    t = translator.translate(text, dest='th')
    return t.text

def many(texts, to='th'):
    translations = translator.translate(texts, dest=to)
    return [t.text for t in translations]



"""
Install manimlib and jupyter-manim, and config them in Colab
"""

import os

os.system("apt install libcairo2-dev libgif-dev")
os.system("apt install texlive-latex-recommended texlive-fonts-extra texlive-science")
os.system("pip install jupyter-manim")

from manimlib.imports import *
import jupyter_manim


# Reduce quality to half of LOW
LOW_QUALITY_CAMERA_CONFIG.update({'pixel_height': 240, 'pixel_width': 426})
defaults = get_ipython().magics_manager.registry['ManimMagics'].defaults
defaults.update({'height': 240, 'width': 426, 'remote': True})
# Without -l, it will use production Q. So, make it LOW by default
PRODUCTION_QUALITY_CAMERA_CONFIG.update(LOW_QUALITY_CAMERA_CONFIG)



"""
Convert a 2D image of a person to a 3D mesh
"""

import os
import torch
import cv2
import numpy as np
from urllib.request import urlretrieve
from kora import trimesh

# a test image (test.png) and default input directory
os.system("npx degit facebookresearch/pifuhd/sample_images sample_images")
image_path = '/content/sample_images/test.png'  # always use this (overwrite)


# pose estimation to crop image
print('Installing pose estimation.')
os.system("npx degit Daniil-Osokin/lightweight-human-pose-estimation.pytorch pose-est")
os.chdir('/content/pose-est')
from models.with_mobilenet import PoseEstimationWithMobileNet
from modules.keypoints import extract_keypoints, group_keypoints
from modules.load_state import load_state
from modules.pose import Pose, track_poses
import demo
# pretrained pose model
print("Downloading pose estimation model.")
url = 'https://download.01.org/opencv/openvino_training_extensions/models/human_pose_estimation/checkpoint_iter_370000.pth'
os.system(f'wget {url}')
# load it
net = PoseEstimationWithMobileNet()
checkpoint = torch.load('checkpoint_iter_370000.pth', map_location='cpu')
load_state(net, checkpoint)

def get_rect(net, images, height_size):
    net = net.eval()

    stride = 8
    upsample_ratio = 4
    num_keypoints = Pose.num_kpts
    previous_poses = []
    delay = 33
    for image in images:
        rect_path = image.replace('.%s' % (image.split('.')[-1]), '_rect.txt')
        img = cv2.imread(image, cv2.IMREAD_COLOR)
        orig_img = img.copy()
        orig_img = img.copy()
        heatmaps, pafs, scale, pad = demo.infer_fast(net, img, height_size, stride, upsample_ratio, cpu=False)

        total_keypoints_num = 0
        all_keypoints_by_type = []
        for kpt_idx in range(num_keypoints):  # 19th for bg
            total_keypoints_num += extract_keypoints(heatmaps[:, :, kpt_idx], all_keypoints_by_type, total_keypoints_num)

        pose_entries, all_keypoints = group_keypoints(all_keypoints_by_type, pafs, demo=True)
        for kpt_id in range(all_keypoints.shape[0]):
            all_keypoints[kpt_id, 0] = (all_keypoints[kpt_id, 0] * stride / upsample_ratio - pad[1]) / scale
            all_keypoints[kpt_id, 1] = (all_keypoints[kpt_id, 1] * stride / upsample_ratio - pad[0]) / scale
        current_poses = []

        rects = []
        for n in range(len(pose_entries)):
            if len(pose_entries[n]) == 0:
                continue
            pose_keypoints = np.ones((num_keypoints, 2), dtype=np.int32) * -1
            valid_keypoints = []
            for kpt_id in range(num_keypoints):
                if pose_entries[n][kpt_id] != -1.0:  # keypoint was found
                    pose_keypoints[kpt_id, 0] = int(all_keypoints[int(pose_entries[n][kpt_id]), 0])
                    pose_keypoints[kpt_id, 1] = int(all_keypoints[int(pose_entries[n][kpt_id]), 1])
                    valid_keypoints.append([pose_keypoints[kpt_id, 0], pose_keypoints[kpt_id, 1]])
            valid_keypoints = np.array(valid_keypoints)
            
            if pose_entries[n][10] != -1.0 or pose_entries[n][13] != -1.0:
              pmin = valid_keypoints.min(0)
              pmax = valid_keypoints.max(0)

              center = (0.5 * (pmax[:2] + pmin[:2])).astype(np.int)
              radius = int(0.65 * max(pmax[0]-pmin[0], pmax[1]-pmin[1]))
            elif pose_entries[n][10] == -1.0 and pose_entries[n][13] == -1.0 and pose_entries[n][8] != -1.0 and pose_entries[n][11] != -1.0:
              # if leg is missing, use pelvis to get cropping
              center = (0.5 * (pose_keypoints[8] + pose_keypoints[11])).astype(np.int)
              radius = int(1.45*np.sqrt(((center[None,:] - valid_keypoints)**2).sum(1)).max(0))
              center[1] += int(0.05*radius)
            else:
              center = np.array([img.shape[1]//2,img.shape[0]//2])
              radius = max(img.shape[1]//2,img.shape[0]//2)

            x1 = center[0] - radius
            y1 = center[1] - radius

            rects.append([x1, y1, 2*radius, 2*radius])

        np.savetxt(rect_path, np.array(rects), fmt='%d')

# get_rect(net.cuda(), [image_path], 512)


# done with get_rect, now chdir back
os.chdir('/content')

# install pifuhd
print("Installing PIFuHD.")
os.system("npx degit facebookresearch/pifuhd/apps apps")
os.system("npx degit facebookresearch/pifuhd/lib lib")
# download pre-trained pifuhd
print("Downloading PIFuHD model.")
os.system("mkdir checkpoints")
url = "https://dl.fbaipublicfiles.com/pifuhd/checkpoints/pifuhd.pt"
os.system(f"wget -P checkpoints {url}")


# !python -m apps.simple_test -r 256 --use_rect
def open(fname=None):
    if fname:
        if fname.startswith('http'):
            urlretrieve(fname, image_path)
        else:
            os.system(f'cp {fname} {image_path}')
    # else use the default test.png
    # crop using pose estimation
    get_rect(net.cuda(), [image_path], 512)
    # finally generate 3d mesh
    os.system("python -m apps.simple_test -r 256 --use_rect")
    mesh = trimesh.load('/content/results/pifuhd_final/recon/result_test_256.obj')
    return mesh



"""
Set default to pyngrok.ngrok
- set auth token automatically, if available
- set connect() to return tunnel, and use https.
"""
import os
from fastcore.utils import partialler

# install pyngrok
os.system('pip install pyngrok')
from pyngrok.ngrok import *


# auto-cofig token
token_file = '/content/drive/My Drive/ngrok.token'
if os.path.exists(token_file):
    with open(token_file) as f:
        token = f.read()
    set_auth_token(token)


# set default arguments for connect
# this behaves like upcoming pyngrok 5.0 and avoid warnings
connect = partialler(connect, return_ngrok_tunnel=True, bind_tls=True)



"""
Convenient tools that call gcloud or gsutil
or work with Google Cloud Platform
"""
import os
from os.path import basename
from IPython import get_ipython

sh = get_ipython().system


def config(project='kora-id'):
    sh(f"gcloud config set project {project}")


def login():
    sh('gcloud auth login')


def list_projects():
    """ List all projects associated with this account """
    sh('gcloud projects list')


def list_services(available=False):
    """ List enabled services. Or all available services.
    """
    cmd = "gcloud services list"
    if available:
        cmd += " --available"
    sh(cmd)


def enable(service):
    """ enable service, which can be listed above """
    if not service.endswith('.googleapis.com'):
        service += '.googleapis.com'
    sh(f'gcloud services enable {service}')


def upload(fname, gs='gs://kora-data'):
    """ Target can be your own gcs, or a subdir inside it.
    Default target is kora's gcs which is publicly writable (auto-delete everyday)
    """
    gs = gs.rstrip('/')
    target = f'{gs}/{basename(fname)}'
    sh(f"gsutil cp {fname} {target}")
    return target



"""
Use docker in Colab, by manual pull and extract
"""

import os
import json
import tarfile
from IPython import get_ipython
from kora.tree import convert, _TEMPLATE

# install docker-pull
os.system("wget https://raw.githubusercontent.com/sdenel/docker-pull/master/docker-pull")
os.system("chmod +x docker-pull")
os.system("mv docker-pull /usr/local/bin")


def pull(fullname):
    if fullname.count('/')==0:
        fullname = 'library/'+fullname
    if fullname.count('/')==1:
        fullname = 'index.docker.io/'+fullname
    target = '/tmp/' + fullname.split('/')[-1]
    cmd = f'docker-pull {fullname} {target}'
    print('> '+cmd)
    get_ipython().system(cmd)  # show progress
    return DockerTar(target)

  
class DockerTar:
    """
    Manage extraction and visualization of a docker tarfile
    Usage:
    > pkg = docker.pull('org/pkg')
    > pkg.extract('*', target='/')
    """
    def __init__(self, file_path):
        self.path = file_path
        self.extract_config()
        self.extract_viz()
    
    def extract_config(self):
        """ get config and layers"""
        tar = tarfile.open(self.path)
        mf = json.load(tar.extractfile('./manifest.json'))[0]
        cfg = './' + mf['Config']
        self.config = json.load(tar.extractfile(cfg))
        self.layers = [l.split('/')[0] for l in mf['Layers']]
        tar.close()

    def extract_viz(self):
        root = {}
        prefix = '/root/.docker-pull-layers-cache/sha256_'
        for layer in self.layers:
            tar = tarfile.open(prefix + layer)
            root[layer] = {}
            for mem in tar.getmembers():
                if mem.isdir(): continue
                node = root[layer]
                for part in mem.name.split('/'):
                    if part not in node:
                        node[part] = {}
                    node = node[part]
            tar.close()
        data = convert(root, False)
        self.viz_data = json.dumps(data)

    def _repr_html_(self):
        """ fancytree of docker layers """
        return _TEMPLATE % self.viz_data

    def extract(self, pattern, target='.', lay=None):
        """
        Extract files by pattern (can be just a file path)
        They are copied to a target directory
        If `lay` is given, it is used to extract from that layer by its prefix.
        """
        for layer in self.layers:
            if lay and not layer.startswith(lay):
                continue  # only look at this layer, if given
            self.extract_layer(layer, pattern, target)

    def extract_layer(self, layer, pattern, target):
        prefix = '/root/.docker-pull-layers-cache/sha256_'
        os.system(f"tar xf {prefix+layer} -C {target} --wildcards '{pattern}' ")



import os

os.system("pip install pythainlp")
os.system("pip install guidedlda")
os.system("pip install pyldavis")

import numpy as np
from pythainlp import word_tokenize
from sklearn.feature_extraction.text import CountVectorizer
from guidedlda import GuidedLDA
import pyLDAvis 
from pyLDAvis._display import prepared_data_to_html
from fastcore.foundation import patch

# avoid warnings, INFO
from warnings import simplefilter
simplefilter(action='ignore', category=FutureWarning)  # in model.fit()

from logging import getLogger, ERROR
getLogger('guidedlda').setLevel(ERROR)
getLogger('numexpr.utils').setLevel(ERROR)


def fit(texts, n_topics=5, n_iter=500, seed=None, seed_confidence=0.15, **kwargs):
    vzer = CountVectorizer(tokenizer=word_tokenize)
    X = vzer.fit_transform(texts)
    vocab = vzer.get_feature_names()
    word2id = vzer.vocabulary_
    opts = get_opts(seed, word2id, seed_confidence)
    # create model
    model = GuidedLDA(n_topics=n_topics, n_iter=n_iter, **kwargs)
    model.fit(X, **opts)  # optional seed
    prepare(model, X, vocab)
    return model


def get_opts(seed, word2id, seed_confidence):
    opts = {}
    if seed:  # seed_topic_list e.g. [[w1,w2], [w3,w4]]
        seed_topics = {}   
        for t_id, st in enumerate(seed):
            for word in st:
                seed_topics[word2id[word]] = t_id
        opts['seed_topics'] = seed_topics
        opts['seed_confidence'] = seed_confidence
    return opts


def _norm(dists):
    return dists / dists.sum(axis=1)[:, None]

def prepare(model, X, vocab):
    doc_lengths = X.sum(axis=1).getA1().tolist()
    term_frequency = X.sum(axis=0).getA1().tolist()
    doc_topic_dists = _norm(model.transform(X)).tolist()
    topic_term_dists = _norm(model.components_).tolist()
    # to be displayed later
    model.viz_data = dict(vocab=vocab,
                          doc_lengths=doc_lengths,
                          term_frequency=term_frequency,
                          doc_topic_dists=doc_topic_dists,
                          topic_term_dists=topic_term_dists)


# auto-viz
@patch
def _repr_html_(self: GuidedLDA):
    data = pyLDAvis.prepare(**self.viz_data)
    return prepared_data_to_html(data)

pyLDAvis.enable_notebook()



import sys
from pkg_resources import get_distribution

def get_ver(package):
    return get_distribution(package).version

__version__ = get_ver('kora')


from fastcore.imports import IN_COLAB

# for %edit
if IN_COLAB:
    import kora.files



import os
from IPython import get_ipython
magic = get_ipython().run_line_magic


os.system("curl https://packages.microsoft.com/keys/microsoft.asc | apt-key add -")
os.system('add-apt-repository "$(curl https://packages.microsoft.com/config/ubuntu/18.04/mssql-server-2019.list)"')
os.system("curl https://packages.microsoft.com/config/ubuntu/18.04/prod.list > /etc/apt/sources.list.d/msprod.list")
os.system("apt update")

os.system("apt install mssql-server")  # install, config, and start
os.environ['MSSQL_PID'] = 'Developer'
os.environ['ACCEPT_EULA'] = 'Y'
os.environ['MSSQL_SA_PASSWORD'] = 'Password.'  # UC, LC, sym or num
os.system("/opt/mssql/bin/mssql-conf -n setup")
os.system("sudo -u mssql /opt/mssql/bin/sqlservr &") # in background

# client
os.system("apt install mssql-tools unixodbc-dev")  # for sqlcmd
os.environ['PATH'] += ':/opt/mssql-tools/bin'
os.system("pip install ipython-sql -U")  # 4.0
os.system("pip install pymssql")   # cannot use pyodbc
magic('load_ext', 'sql')
magic('config', 'SqlMagic.displaycon=False')
magic('config', 'SqlMagic.autopandas=True')
# magic('config', 'SqlMagic.feedback=False')  # Done, rows affected

# default connection
magic('sql', 'mssql+pymssql://sa:Password.@localhost:1433/master')



"""
Kaldi is a library for Speech Recognition
"""
import os

# Just extract from my custom tar. Very fast (17s)
url = 'https://github.com/airesearch-in-th/kora/releases/download/v0.7/kaldi.tar.gz'
os.system(f"curl -L {url} | tar xz -C /")

# Add paths to binaries
paths = [''] + """
/opt/kaldi/src/bin
/opt/kaldi/src/featbin
/opt/kaldi/src/fstbin
/opt/kaldi/src/gmmbin
/opt/kaldi/tools/openfst/bin
""".split()

os.environ['PATH'] += ':'.join(paths)



"""
Python wrapper around `thpronun` (https://github.com/tlwg/thpronun)
Many thanks to P' Thep and P' Ott
"""
import os
import subprocess
from ast import literal_eval


# install thpronun to /usr/local/bin
url = 'https://github.com/airesearch-in-th/kora/releases/download/v0.7/thpronun.tar'
os.system(f'curl -L {url} | tar x -C /')


output_dict = {
    'roman': 'r',
    'thai': 't',
    'phonetic': 'p',
    'raw': 'w',
    'soundex': 's',
}

def g2p(word, output='phonetic', group=False):
    """ 
    From word to pronunication. You can set `output` to 
    - phonetic (default: equivalent to IPA)
    - roman (Royin romanization)
    - thai (Thai pronunication, easy to read)
    - soundex (for similarity search)
    - raw (internal format)

    For a long sentence, you may want to set `group` to True 
    to avoid too many possibilities.
    """
    output = output_dict.get(output, output)
    cmd = ['thpronun', '-'+output]
    if group:
        cmd.append('-g')
    cmd.append(word)
    res = subprocess.check_output(cmd)
    lines = res.decode().strip().split('\n')
    if group:
        return literal_eval(lines[1])
    else:
        return lines[1:]



""" What I want to add to pandas 

Initially just ability to read and write to google sheet
"""

import os
import re
from pandas import *
import gspread_dataframe


def auth_gsheet():
    from google.colab import auth
    auth.authenticate_user()
    import gspread
    from oauth2client.client import GoogleCredentials as GC
    gsheet = gspread.authorize(GC.get_application_default())
    return gsheet


def gsheet_open(source):
    """ Open existing title/id/url, or create a new one
    
    Return the first sheet (sheet1)
    """
    g = auth_gsheet()
    # a URL ?
    if source.startswith('http'):
        return g.open_by_url(source).sheet1
    # a file_id ?
    if re.match(r'1[-\w]{43}$', source):
        return g.open_by_key(source).sheet1
    # otherwise, it's a title
    try:
        return g.open(source).sheet1
    except:
        return g.create(source).sheet1


def read_sheet(source, **kw):
    """ Imitate pd.read_csv()
    
    Source can be title, fild_id, or URL
    """
    sheet = gsheet_open(source)
    return (gspread_dataframe
            .get_as_dataframe(sheet, **kw)
            .dropna(0, 'all').dropna(1, 'all')
            .fillna(''))
    

def _df_to_sheet(self, title, 
                 header=True, index=False, resize=True, 
                 public=False):
    sheet = gsheet_open(title)
    (gspread_dataframe
     .set_with_dataframe(sheet, self,
                        include_index=index,
                        include_column_header=header,
                        resize=resize)
    )
    if public:
        sheet.spreadsheet.share(None, 'anyone', 'reader')
    return 'https://docs.google.com/spreadsheets/d/' + sheet.spreadsheet.id

DataFrame.to_sheet = _df_to_sheet


# Notion.so

def get_notion_client(token=None):
    # Lazy install notion-py if not installed
    try: 
        from notion.client import NotionClient
    except:
        os.system("pip install notion")
        from notion.client import NotionClient

    # Allow access to public notion without token
    class NotionWrapper(NotionClient):
        def _update_user_info(self):
            pass
    # So, use Wrapper instead of NotionClient
    return NotionWrapper(token_v2=token)


def read_notion(url, **kwargs):
    """ Just like read_csv """
    client = get_notion_client()
    cv = client.get_collection_view(url)
    data = [row.get_all_properties() for row in cv.collection.get_rows()]
    return DataFrame(data, **kwargs)


def to_interval(istr):
    """
    Convert from string to pd.Interval
    Useful for reading a csv with IntervalIndex
       pd.read_csv(path, index_col=0, converters={0: to_interval})
    """
    c_left = istr[0]=='['
    c_right = istr[-1]==']'
    closed = {(True, False): 'left',
              (False, True): 'right',
              (True, True): 'both',
              (False, False): 'neither'
             }[c_left, c_right]
    left, right = map(float, istr[1:-1].split(','))
    return Interval(left, right, closed)



"""
InfluxDB is a time-series database
"""
import os

# install server
os.system("curl -sL https://repos.influxdata.com/influxdb.key | apt-key add -")
with open('/etc/apt/sources.list.d/influxdb.list', 'w') as f:
    f.write("deb https://repos.influxdata.com/ubuntu bionic stable")
os.system('apt update')
os.system('apt install influxdb')
os.system('influxd -config /etc/influxdb/influxdb.conf &')

# install client
os.system("pip install influxdb")



"""
Papers with Code dataset
For tracking the lastest AI research papers
"""

import os

urls = """
https://paperswithcode.com/media/about/papers-with-abstracts.json.gz
https://paperswithcode.com/media/about/links-between-papers-and-code.json.gz
https://paperswithcode.com/media/about/evaluation-tables.json.gz
https://paperswithcode.com/media/about/methods.json.gz
""".split()

def download():
    """ downloand and unzip into PPWC """
    os.mkdir('PPWC')
    for url in urls:
        fname = url.split('/')[-1]
        jname = fname[:-3]
        os.system(f"wget {url}")
        os.system(f"gunzip {fname}")
        os.system(f"mv {jname} PPWC")



"""
Shortcuts for tensorflow_hub
Instead of `import tensorflow_hub as hub`, call `from kora import hub`.
You will get hub.embed for Universal Sentence Encoder.
There will be more shortcuts for hub in the future.
"""
from tensorflow_hub import * 


_embed = None  # for lazy loading
use_url = "https://tfhub.dev/google/universal-sentence-encoder/4"

def load_use():
    """ lazy load and cache it """
    global _embed
    if _embed is None:
        _embed = load(use_url)
    return _embed


def embed(input):
    """ embed 1 sentence, or a list of sentence """
    if type(input) == str:
        return load_use()([input])[0]   # just 1
    else:
        return load_use()(input)



"""
__init__.py contains only small viz I use frequently
"""
import pandas as pd
from collections import Counter

def histogram(data, name='num', **kw):
    """
    No binning, just a bar chart of Counter()
    **kw can be title, width, or range_x=(0,10)
    """
    import plotly.express as px
    items = Counter(data).items()
    df = pd.DataFrame(items, columns=[name, 'freq'])
    return px.bar(df, x=name, y='freq', **kw)



"""
From InstColorization
Github: github.com/ericsujw/InstColorization
Colab: colab.to/ericsujw/InstColorization/InstColorization.ipynb

Simplified into a function call
"""
import os
from os import listdir
from os.path import join, isfile, isdir
from urllib.request import urlretrieve
import sys
import numpy as np
import cv2
import PIL
import torch

import warnings

warnings.simplefilter("ignore")
os.environ["CUDA_VISIBLE_DEVICES"] = "0"


# Install InstColorization
print("Installing InstColorization")
os.system("npx degit ericsujw/InstColorization inst")
sys.path.append('/content/inst')  # allow import
# Load checkpoint
print("Loading Colorization model")
os.system("wget https://github.com/airesearch-in-th/kora/releases/download/v0.9/colorization_checkpoints.zip")
os.system("unzip colorization_checkpoints.zip")
os.system("rm colorization_checkpoints.zip")


# Install Detectron
print("Installing Detectron2")
os.system("pip install pyyaml -U")  # 5.1+
os.system("pip install detectron2 -f https://dl.fbaipublicfiles.com/detectron2/wheels/cu101/index.html")
PIL.TiffTags.IFD = 13   # instead of restart

print("Loading Detectron2 model")
import detectron2
from detectron2.utils.logger import setup_logger
setup_logger()

# import some common detectron2 utilities
from detectron2 import model_zoo
from detectron2.engine import DefaultPredictor
from detectron2.config import get_cfg

cfg = get_cfg()
cfg.merge_from_file(model_zoo.get_config_file("COCO-InstanceSegmentation/mask_rcnn_X_101_32x8d_FPN_3x.yaml"))
cfg.MODEL.ROI_HEADS.SCORE_THRESH_TEST = 0.7
cfg.MODEL.WEIGHTS = model_zoo.get_checkpoint_url("COCO-InstanceSegmentation/mask_rcnn_X_101_32x8d_FPN_3x.yaml")
predictor = DefaultPredictor(cfg)


# Create directories
for d in 'example example_bbox results'.split():
  os.makedirs(d, exist_ok=True)
input_dir = "example"
output_npz_dir = "example_bbox"


# Importing InstColorization
from options.train_options import TestOptions
from models import create_model
from fusion_dataset import Fusion_Testing_Dataset
from util import util
import multiprocessing
multiprocessing.set_start_method('spawn', True)

torch.backends.cudnn.benchmark = True

sys.argv = [sys.argv[0]]  # clear options ?
opt = TestOptions().parse()
save_img_path = opt.results_img_dir

model = create_model(opt)
model.setup_to_test('coco_finetuned_mask_256_ffs')


def open(fname):
    # copy to input
    input_img = 'example/input.jpg'
    if fname.startswith('http'):
        urlretrieve(fname, input_img)
    else:
        os.system(f'cp {fname} {input_img}')
    # find bbox
    img = cv2.imread(input_img)
    lab_image = cv2.cvtColor(img, cv2.COLOR_BGR2LAB)
    l_channel, a_channel, b_channel = cv2.split(lab_image)
    l_stack = np.stack([l_channel, l_channel, l_channel], axis=2)
    outputs = predictor(l_stack)
    save_path = 'example_bbox/input'
    pred_bbox = outputs["instances"].pred_boxes.to(torch.device('cpu')).tensor.numpy()
    pred_scores = outputs["instances"].scores.cpu().data.numpy()
    np.savez(save_path, bbox = pred_bbox, scores = pred_scores)
    # load data
    dataset = Fusion_Testing_Dataset(opt, -1)
    dataset_loader = torch.utils.data.DataLoader(dataset, batch_size=opt.batch_size)
    data_raw = next(iter(dataset_loader))
    # predict colors
    data_raw['full_img'][0] = data_raw['full_img'][0].cuda()
    if data_raw['empty_box'][0] == 0:
        data_raw['cropped_img'][0] = data_raw['cropped_img'][0].cuda()
        box_info = data_raw['box_info'][0]
        box_info_2x = data_raw['box_info_2x'][0]
        box_info_4x = data_raw['box_info_4x'][0]
        box_info_8x = data_raw['box_info_8x'][0]
        cropped_data = util.get_colorization_data(data_raw['cropped_img'], opt, ab_thresh=0, p=opt.sample_p)
        full_img_data = util.get_colorization_data(data_raw['full_img'], opt, ab_thresh=0, p=opt.sample_p)
        model.set_input(cropped_data)
        model.set_fusion_input(full_img_data, [box_info, box_info_2x, box_info_4x, box_info_8x])
        model.forward()
    else:
        full_img_data = util.get_colorization_data(data_raw['full_img'], opt, ab_thresh=0, p=opt.sample_p)
        model.set_forward_without_box(full_img_data)
    model.save_current_imgs(join(save_img_path, data_raw['file_id'][0] + '.png'))
    # combine grey & color
    img = cv2.imread('example/input.jpg')
    lab_image = cv2.cvtColor(img, cv2.COLOR_BGR2LAB)
    l_channel, _, _ = cv2.split(lab_image)

    img = cv2.imread('results/input.png')
    lab_image = cv2.cvtColor(img, cv2.COLOR_BGR2LAB)
    _, a_pred, b_pred = cv2.split(lab_image)
    a_pred = cv2.resize(a_pred, (l_channel.shape[1], l_channel.shape[0]))
    b_pred = cv2.resize(b_pred, (l_channel.shape[1], l_channel.shape[0]))

    color_image = cv2.cvtColor(np.stack([l_channel, a_pred, b_pred], 2), cv2.COLOR_LAB2BGR)
    return PIL.Image.fromarray(color_image[:,:,::-1])  # BGR -> RGB



"""
TSync2 Speech Corpus for TTS, by NECTEC
Licensed under CC-BY-NC-SA
"""

import os


def download():
    url = "https://github.com/korakot/corpus/releases/download/v1.0/AIFORTHAI-TSync2Corpus.zip"
    print("NECTEC licensed TSync2 under CC-BY-NC-SA")
    print("Start downloading: .. ")
    os.system(f"wget {url}")
    os.system("unzip AIFORTHAI-TSync2Corpus.zip")
    os.system("rm AIFORTHAI-TSync2Corpus.zip")
    print("Finished")



"""
Use AIResearch MT model easily
"""

import os

os.system('pip install sentencepiece')
os.system('pip install git+https://github.com/pytorch/fairseq@6f6461b')

from fairseq.models.transformer import TransformerModel


# download model
url = 'https://github.com/vistec-AI/model-releases/releases/download/SCB_1M%2BTBASE_v1.0/SCB_1M-MT_OPUS+TBASE_th-en_spm-spm_32000-joined_v1.0.tar.gz'
os.system(f'curl -L {url} | tar xz')

model = TransformerModel.from_pretrained(
    model_name_or_path='SCB_1M-MT_OPUS+TBASE_th-en_spm-spm_32000-joined_v1.0/models/',
    checkpoint_file='checkpoint.pt',
    data_name_or_path='SCB_1M-MT_OPUS+TBASE_th-en_spm-spm_32000-joined_v1.0/vocab/',
    bpe='sentencepiece',
    sentencepiece_vocab='SCB_1M-MT_OPUS+TBASE_th-en_spm-spm_32000-joined_v1.0/bpe/spm.th.model'
)

# function en2th.translate
translate = model.translate



import os
import re
import requests 
from subprocess import Popen, PIPE, STDOUT
from IPython.core.magic import register_cell_magic
from urllib.parse import urljoin


# download server
ver = '7.9.2'
url = f'https://artifacts.elastic.co/downloads/elasticsearch/elasticsearch-{ver}-linux-x86_64.tar.gz'
os.system(f"curl {url} | tar xz")
os.system(f"chown -R daemon:daemon elasticsearch-{ver}")

# start server
es_server = Popen([f'elasticsearch-{ver}/bin/elasticsearch'], 
                  stdout=PIPE, stderr=STDOUT,
                  preexec_fn=lambda: os.setuid(1)  # as daemon
                 )
for i in range(77):
    es_server.stdout.readline()  # wait till started


# client
os.system("pip install elasticsearch")

# client magic
@register_cell_magic
def es(line=None, cell=""):
    cell = re.sub(r'(?m)^\s*#.*\n?','', cell) # remove comment
    line1 = (cell + '\n').find('\n')
    method, path = cell[:line1].split(None, 1)
    body = cell[line1:].strip()
    args = {}
    if body:
        args['data'] = (body + '\n').encode()  # add \n in case _bulk
        args['headers'] = {'Content-Type': 'application/json'}

    url = urljoin('http://localhost:9200', path)
    req = requests.Request(method, url, **args).prepare()
    rsp = requests.Session().send(req)
    return rsp

# client viz
def render(r):
    text = r.text  
    if text[0] in "[{":  # really json
        return """
        <script src="https://rawgit.com/caldwell/renderjson/master/renderjson.js"></script>
        <script>
        renderjson.set_show_to_level(1)
        document.body.appendChild(renderjson(%s))
        new ResizeObserver(google.colab.output.resizeIframeToContent).observe(document.body) 
        </script>
        """ % text
    else:    # other status text
        return "<pre>%s</pre>" % text
requests.models.Response._repr_html_ = render



""" What bs4 should have been """
import os
os.system("pip install beautifulsoup4 -U") # 4.9 for advanced selector
from bs4 import BeautifulSoup, Tag
import requests 
from pathlib import Path


def Soup(s, features='lxml', **kw):
    """ A fake class, of what BeautifulSoup should have been 
    
    It accepts a url or a file, in addition to html/xml as usual
    """
    if isinstance(s, Path):
        src = s.read_text()
    elif s.startswith('http'):
        src = requests.get(s).text
    elif Path(s).exists():
        src = open(s).read()
    else:
        src = s
    return BeautifulSoup(src, features, **kw)


Tag.select1 = Tag.select_one
